@@ -3,16 +3,22 @@
 #include "markup.h"
 
 #include <assert.h>
+#include <ctype.h>
 #include <stdbool.h>
-#include <string.h>
 #include <stdio.h>
+#include <string.h>
 
+#include "log.h"
 #include "settings.h"
 #include "utils.h"
 
+/**
+ * Convert all HTML special symbols to HTML entities.
+ * @param str (nullable)
+ */
 static char *markup_quote(char *str)
 {
-        assert(str != NULL);
+        ASSERT_OR_RET(str, NULL);
 
         str = string_replace_all("&", "&amp;", str);
         str = string_replace_all("\"", "&quot;", str);
@@ -23,9 +29,13 @@ static char *markup_quote(char *str)
         return str;
 }
 
+/**
+ * Convert all HTML special entities to their actual char.
+ * @param str (nullable)
+ */
 static char *markup_unquote(char *str)
 {
-        assert(str != NULL);
+        ASSERT_OR_RET(str, NULL);
 
         str = string_replace_all("&quot;", "\"", str);
         str = string_replace_all("&apos;", "'", str);
@@ -36,9 +46,13 @@ static char *markup_unquote(char *str)
         return str;
 }
 
+/**
+ * Convert all HTML linebreak tags to a newline character
+ * @param str (nullable)
+ */
 static char *markup_br2nl(char *str)
 {
-        assert(str != NULL);
+        ASSERT_OR_RET(str, NULL);
 
         str = string_replace_all("<br>", "\n", str);
         str = string_replace_all("<br/>", "\n", str);
@@ -46,15 +60,10 @@ static char *markup_br2nl(char *str)
         return str;
 }
 
-/*
- * Remove HTML hyperlinks of a string.
- *
- * @str: The string to replace a tags
- * @urls: (nullable): If any href-attributes found, an '\n' concatenated
- *        string of the URLs in format '[<text between tags>] <href>'
- */
+/* see markup.h */
 void markup_strip_a(char **str, char **urls)
 {
+        assert(*str);
         char *tag1 = NULL;
 
         if (urls)
@@ -68,17 +77,15 @@ void markup_strip_a(char **str, char **urls)
 
                 // the tag is broken, ignore it
                 if (!tag1_end) {
-                        fprintf(stderr,
-                                "WARNING: Given link is broken: '%s'\n",
-                                tag1);
+                        LOG_W("Given link is broken: '%s'",
+                              tag1);
                         string_replace_at(*str, tag1-*str, strlen(tag1), "");
                         break;
                 }
                 if (tag2 && tag2 < tag1_end) {
                         int repl_len =  (tag2 - tag1) + strlen("</a>");
-                        fprintf(stderr,
-                                "WARNING: Given link is broken: '%.*s.'\n",
-                                repl_len, tag1);
+                        LOG_W("Given link is broken: '%.*s.'",
+                              repl_len, tag1);
                         string_replace_at(*str, tag1-*str, repl_len, "");
                         break;
                 }
@@ -128,16 +135,10 @@ void markup_strip_a(char **str, char **urls)
         }
 }
 
-/*
- * Remove img-tags of a string. If alt attribute given, use this as replacement.
- *
- * @str: The string to replace img tags
- * @urls: (nullable): If any src-attributes found, an '\n' concatenated string of
- *        the URLs in format '[<alt>] <src>'
- */
+/* see markup.h */
 void markup_strip_img(char **str, char **urls)
 {
-        const char *start = *str;
+        const char *start;
 
         if (urls)
                 *urls = NULL;
@@ -147,7 +148,7 @@ void markup_strip_img(char **str, char **urls)
 
                 // the tag is broken, ignore it
                 if (!end) {
-                        fprintf(stderr, "WARNING: Given image is broken: '%s'\n", start);
+                        LOG_W("Given image is broken: '%s'", start);
                         string_replace_at(*str, start-*str, strlen(start), "");
                         break;
                 }
@@ -188,9 +189,8 @@ void markup_strip_img(char **str, char **urls)
                         text_src = g_strndup(src_s, src_e-src_s);
 
                 } else {
-                        fprintf(stderr,
-                                "WARNING: Given image argument is broken: '%.*s'\n",
-                                (int)(end-start), start);
+                         LOG_W("Given image argument is broken: '%.*s'",
+                               (int)(end-start), start);
                 }
 
                 // replacement text for alt
@@ -218,23 +218,10 @@ void markup_strip_img(char **str, char **urls)
         }
 }
 
-/*
- * Strip any markup from text; turn it in to plain text.
- *
- * For well-formed markup, the following two commands should be
- * roughly equivalent:
- *
- *     out = markup_strip(in);
- *     pango_parse_markup(in, -1, 0, NULL, &out, NULL, NULL);
- *
- * However, `pango_parse_markup()` balks at invalid markup;
- * `markup_strip()` shouldn't care if there is invalid markup.
- */
+/* see markup.h */
 char *markup_strip(char *str)
 {
-        if (str == NULL) {
-                return NULL;
-        }
+        ASSERT_OR_RET(str, NULL);
 
         /* strip all tags */
         string_strip_delimited(str, '<', '>');
@@ -245,15 +232,83 @@ char *markup_strip(char *str)
         return str;
 }
 
-/*
- * Transform the string in accordance with `markup_mode` and
- * `settings.ignore_newline`
+/**
+ * Determine if an & character pointed to by \p str is a markup & entity or
+ * part of the text
+ *
+ * @retval true: \p str is an entity
+ * @retval false: It's no valid entity
  */
+static bool markup_is_entity(const char *str)
+{
+        assert(str);
+        assert(*str == '&');
+
+        char *end = strchr(str, ';');
+        ASSERT_OR_RET(end, false);
+
+        // Parse (hexa)decimal entities with the format &#1234; or &#xABC;
+        if (str[1] == '#') {
+                const char *cur = str + 2;
+
+                if (*cur == 'x') {
+                        cur++;
+
+                        // Reject &#x;
+                        if (*cur == ';')
+                                return false;
+
+                        while (isxdigit(*cur) && cur < end)
+                                cur++;
+                } else {
+
+                        // Reject &#;
+                        if (*cur == ';')
+                                return false;
+
+                        while (isdigit(*cur) && cur < end)
+                                cur++;
+                }
+
+                return (cur == end);
+        } else {
+                const char *supported_tags[] = {"&amp;", "&lt;", "&gt;", "&quot;", "&apos;"};
+                for (int i = 0; i < sizeof(supported_tags)/sizeof(*supported_tags); i++) {
+                        if (g_str_has_prefix(str, supported_tags[i]))
+                                return true;
+                }
+                return false;
+        }
+}
+
+/**
+ * Escape all unsupported and invalid &-entities in a string. If the resulting
+ * string does not fit it will be reallocated.
+ *
+ * @param str The string to be transformed
+ */
+static char *markup_escape_unsupported(char *str)
+{
+        ASSERT_OR_RET(str, NULL);
+
+        char *match = str;
+        while ((match = strchr(match, '&'))) {
+                if (!markup_is_entity(match)) {
+                        int pos = match - str;
+                        str = string_replace_at(str, pos, 1, "&amp;");
+                        match = str + pos + strlen("&amp;");
+                } else {
+                        match++;
+                }
+        }
+
+        return str;
+}
+
+/* see markup.h */
 char *markup_transform(char *str, enum markup_mode markup_mode)
 {
-        if (str == NULL) {
-                return NULL;
-        }
+        ASSERT_OR_RET(str, NULL);
 
         switch (markup_mode) {
         case MARKUP_NULL:
@@ -269,6 +324,7 @@ char *markup_transform(char *str, enum markup_mode markup_mode)
                 str = markup_quote(str);
                 break;
         case MARKUP_FULL:
+                str = markup_escape_unsupported(str);
                 str = markup_br2nl(str);
                 markup_strip_a(&str, NULL);
                 markup_strip_img(&str, NULL);
