@@ -8,7 +8,7 @@
 #include "notification.h"
 #include "settings.h"
 
-typedef struct _rule_t {
+struct rule {
         char *name;
         /* filters */
         char *appname;
@@ -16,6 +16,8 @@ typedef struct _rule_t {
         char *body;
         char *icon;
         char *category;
+        char *stack_tag;
+        char *desktop_entry;
         int msg_urgency;
 
         /* actions */
@@ -25,19 +27,29 @@ typedef struct _rule_t {
         int history_ignore;
         int match_transient;
         int set_transient;
+        int skip_display;
         char *new_icon;
         char *fg;
         char *bg;
+        char *fc;
         const char *format;
         const char *script;
-} rule_t;
+        enum behavior_fullscreen fullscreen;
+        char *set_stack_tag;
+};
 
 extern GSList *rules;
 
-void rule_init(rule_t *r);
-void rule_apply(rule_t *r, notification *n);
-void rule_apply_all(notification *n);
-bool rule_matches_notification(rule_t *r, notification *n);
+/**
+ * Allocate a new rule. The rule is fully initialised.
+ *
+ * @returns A new initialised rule.
+ */
+struct rule *rule_new(void);
+
+void rule_apply(struct rule *r, struct notification *n);
+void rule_apply_all(struct notification *n);
+bool rule_matches_notification(struct rule *r, struct notification *n);
 
 #endif
 /* vim: set tabstop=8 shiftwidth=8 expandtab textwidth=0: */
