@@ -5,12 +5,25 @@
 #include <ctype.h>
 #include <errno.h>
 #include <glib.h>
+#include <pwd.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <time.h>
+#include <unistd.h>
 
 #include "log.h"
 
+/* see utils.h */
+void free_string_array(char **arr)
+{
+        if (arr){
+                for (int i = 0; arr[i]; i++){
+                        g_free(arr[i]);
+                }
+        }
+        g_free(arr);
+}
+
 /* see utils.h */
 char *string_replace_char(char needle, char replacement, char *haystack)
 {
@@ -133,12 +146,25 @@ void string_strip_delimited(char *str, char a, char b)
         str[iwrite] = 0;
 }
 
+/* see utils.h */
+char **string_to_array(const char *string)
+{
+        char **arr = NULL;
+        if (string) {
+                arr = g_strsplit(string, ",", 0);
+                for (int i = 0; arr[i]; i++){
+                        g_strstrip(arr[i]);
+                }
+        }
+        return arr;
+}
+
 /* see utils.h */
 char *string_to_path(char *string)
 {
 
         if (string && STRN_EQ(string, "~/", 2)) {
-                char *home = g_strconcat(getenv("HOME"), "/", NULL);
+                char *home = g_strconcat(user_get_home(), "/", NULL);
 
                 string = string_replace_at(string, 0, 2, home);
 
@@ -204,4 +230,21 @@ gint64 time_monotonic_now(void)
 #endif
         return S2US(tv_now.tv_sec) + tv_now.tv_nsec / 1000;
 }
+
+/* see utils.h */
+const char *user_get_home(void)
+{
+        static const char *home_directory = NULL;
+        ASSERT_OR_RET(!home_directory, home_directory);
+
+        // Check the HOME variable for the user's home
+        home_directory = getenv("HOME");
+        ASSERT_OR_RET(!home_directory, home_directory);
+
+        // Check the /etc/passwd entry for the user's home
+        home_directory = getpwuid(getuid())->pw_dir;
+
+        return home_directory;
+}
+
 /* vim: set tabstop=8 shiftwidth=8 expandtab textwidth=0: */
