@@ -6,6 +6,7 @@
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gio/gio.h>
 
+#include "helpers.h"
 #include "queues.h"
 
 extern const char *base;
@@ -252,33 +253,13 @@ bool dbus_notification_fire(struct dbus_notification *n, uint *id)
 
 void dbus_notification_set_raw_image(struct dbus_notification *n_dbus, const char *path)
 {
-        GdkPixbuf *pb = gdk_pixbuf_new_from_file(path, NULL);
-
-        if (!pb)
+        GVariant *hint = notification_setup_raw_image(path);
+        if (!hint)
                 return;
 
-        GVariant *hint_data = g_variant_new_from_data(
-                                G_VARIANT_TYPE("ay"),
-                                gdk_pixbuf_read_pixels(pb),
-                                gdk_pixbuf_get_byte_length(pb),
-                                TRUE,
-                                (GDestroyNotify) g_object_unref,
-                                g_object_ref(pb));
-
-        GVariant *hint = g_variant_new(
-                                "(iiibii@ay)",
-                                gdk_pixbuf_get_width(pb),
-                                gdk_pixbuf_get_height(pb),
-                                gdk_pixbuf_get_rowstride(pb),
-                                gdk_pixbuf_get_has_alpha(pb),
-                                gdk_pixbuf_get_bits_per_sample(pb),
-                                gdk_pixbuf_get_n_channels(pb),
-                                hint_data);
-
         g_hash_table_insert(n_dbus->hints,
                             g_strdup("image-data"),
                             g_variant_ref_sink(hint));
-        g_object_unref(pb);
 }
 
 /////// TESTS
@@ -660,6 +641,34 @@ TEST test_hint_raw_image(void)
         PASS();
 }
 
+/* We didn't process the timeout parameter via DBus correctly
+ * and it got limited to an int instead of a long int
+ * See: Issue #646 (The timeout value in dunst wraps around) */
+TEST test_timeout_overflow(void)
+{
+        struct notification *n;
+        struct dbus_notification *n_dbus;
+
+        n_dbus = dbus_notification_new();
+        n_dbus->app_name = "dunstteststack";
+        n_dbus->app_icon = "NONE";
+        n_dbus->summary = "test_hint_urgency";
+        n_dbus->body = "Summary of it";
+        n_dbus->expire_timeout = 2147484;
+        gint64 expected_timeout = G_GINT64_CONSTANT(2147484000);
+
+        guint id;
+        ASSERT(dbus_notification_fire(n_dbus, &id));
+        ASSERT(id != 0);
+
+        n = queues_debug_find_notification_by_id(id);
+        ASSERT_EQ_FMT(expected_timeout, n->timeout, "%" G_GINT64_FORMAT);
+
+        dbus_notification_free(n_dbus);
+
+        PASS();
+}
+
 TEST test_server_caps(enum markup_mode markup)
 {
         GVariant *reply;
@@ -790,6 +799,12 @@ TEST assert_methodlists_sorted(void)
                                 methods_fdn[i+1].method_name));
         }
 
+        for (size_t i = 0; i+1 < G_N_ELEMENTS(methods_dunst); i++) {
+                ASSERT(0 > strcmp(
+                                methods_dunst[i].method_name,
+                                methods_dunst[i+1].method_name));
+        }
+
         PASS();
 }
 
@@ -821,6 +836,7 @@ gpointer run_threaded_tests(gpointer data)
         RUN_TESTp(test_server_caps, MARKUP_NO);
         RUN_TEST(test_close_and_signal);
         RUN_TEST(test_signal_actioninvoked);
+        RUN_TEST(test_timeout_overflow);
 
         RUN_TEST(assert_methodlists_sorted);
 
