@@ -9,9 +9,12 @@ make -C "${BASE}" SYSTEMD=1 SERVICEDIR_SYSTEMD="${PREFIX}/systemd" SERVICEDIR_DB
 
 diff -u <(find "${PREFIX}" -type f -printf "%P\n" | sort) - <<EOF
 bin/dunst
+bin/dunstctl
+bin/dunstify
 dbus/org.knopwob.dunst.service
 share/dunst/dunstrc
 share/man/man1/dunst.1
+share/man/man1/dunstctl.1
 systemd/dunst.service
 EOF
 
