@@ -1,11 +1,10 @@
+#include "../src/utils.c"
 #include "greatest.h"
-#include "src/utils.h"
-
-#include <glib.h>
 
 TEST test_string_replace_char(void)
 {
-        char *text = malloc(128 * sizeof(char));
+        char *text = g_malloc(128 * sizeof(char));
+
         strcpy(text, "a aa aaa");
         ASSERT_STR_EQ("b bb bbb", string_replace_char('a', 'b', text));
 
@@ -14,8 +13,8 @@ TEST test_string_replace_char(void)
 
         strcpy(text, "");
         ASSERT_STR_EQ("", string_replace_char('a', 'b', text));
-        free(text);
 
+        g_free(text);
         PASS();
 }
 
@@ -27,8 +26,8 @@ TEST test_string_replace_char(void)
 
 TEST test_string_replace_all(void)
 {
+        char *text = g_malloc(128 * sizeof(char));
 
-        char *text = malloc(128 * sizeof(char));
         strcpy(text, "aaaaa");
         ASSERT_STR_EQ("bbbbb", (text = string_replace_all("a", "b", text)));
 
@@ -44,29 +43,7 @@ TEST test_string_replace_all(void)
         strcpy(text, "abcdabc");
         ASSERT_STR_EQ("xyzabcdxyzabc", (text = string_replace_all("a", "xyza", text)));
 
-        free(text);
-        PASS();
-}
-
-TEST test_string_replace(void)
-{
-        char *text = malloc(128 * sizeof(char));
-        strcpy(text, "aaaaa");
-        ASSERT_STR_EQ("baaaa", (text = string_replace("a", "b", text)) );
-
-        strcpy(text, "");
-        ASSERT_STR_EQ((text = string_replace("a", "b", text)), "");
-
-        strcpy(text, "Nothing to replace");
-        ASSERT_STR_EQ((text = string_replace("z", "a", text)), "Nothing to replace");
-
-        strcpy(text, "Reverse this");
-        ASSERT_STR_EQ("Reverse sith", (text = string_replace("this", "sith", text)));
-
-        strcpy(text, "abcdabc");
-        ASSERT_STR_EQ("xyzabcdabc", (text = string_replace("a", "xyza", text)));
-
-        free(text);
+        g_free(text);
         PASS();
 }
 
@@ -104,9 +81,35 @@ TEST test_string_append(void)
         PASS();
 }
 
+TEST test_string_strip_quotes(void)
+{
+        char *exp = string_strip_quotes(NULL);
+        ASSERT_FALSE(exp);
+
+        ASSERT_STR_EQ("NewString", (exp = string_strip_quotes("NewString")));
+        g_free(exp);
+
+        ASSERT_STR_EQ("becomes unquoted", (exp = string_strip_quotes("\"becomes unquoted\"")));
+        g_free(exp);
+
+        ASSERT_STR_EQ("\"stays quoted", (exp = string_strip_quotes("\"stays quoted")));
+        g_free(exp);
+
+        ASSERT_STR_EQ("stays quoted\"", (exp = string_strip_quotes("stays quoted\"")));
+        g_free(exp);
+
+        ASSERT_STR_EQ("stays \"quoted\"", (exp = string_strip_quotes("stays \"quoted\"")));
+        g_free(exp);
+
+        ASSERT_STR_EQ(" \"stays quoted\"", (exp = string_strip_quotes(" \"stays quoted\"")));
+        g_free(exp);
+
+        PASS();
+}
+
 TEST test_string_strip_delimited(void)
 {
-        char *text = malloc(128 * sizeof(char));
+        char *text = g_malloc(128 * sizeof(char));
 
         strcpy(text, "A <simple> string_strip_delimited test");
         string_strip_delimited(text, '<', '>');
@@ -128,7 +131,7 @@ TEST test_string_strip_delimited(void)
         string_strip_delimited(text, '<', '>');
         ASSERT_STR_EQ("Nothing is done if there are no delimiters in the string", text);
 
-        free(text);
+        g_free(text);
         PASS();
 }
 
@@ -176,8 +179,8 @@ SUITE(suite_utils)
 {
         RUN_TEST(test_string_replace_char);
         RUN_TEST(test_string_replace_all);
-        RUN_TEST(test_string_replace);
         RUN_TEST(test_string_append);
+        RUN_TEST(test_string_strip_quotes);
         RUN_TEST(test_string_strip_delimited);
         RUN_TEST(test_string_to_path);
         RUN_TEST(test_string_to_time);
